# -*- encoding: utf-8 -*- 

import pygame
from HamstersAndPlague.Constants import *
import pkg_resources
import os, os.path


def LoadImage(folder, file):
	path = folder + "/" + file
	
	try:
		image = pygame.image.load(pkg_resources.resource_stream("HamstersAndPlague", path))
	except pygame.error, e:
		raise SystemExit, e
	
	image = image.convert_alpha()
	
	return image	

def LoadSound(folder, file):
	class NoneSound:
		def play(self): pass
	
	if not pygame.mixer:
		return NoneSound()
	
	path = folder + "/" + file
	try:
		sound = pygame.mixer.Sound(pkg_resources.resource_stream("HamstersAndPlague", path))
	except pygame.error, e:
		raise SystemExit, e
	
	return sound
	
	
def LoadMusic(folder, file):
	path = folder + "/" + file
	try:
		pygame.mixer.music.load(pkg_resources.resource_stream("HamstersAndPlague", path))
	except pygame.error, e:
		pass
	


class BackgroundController:
	
	@classmethod
	def Load(cls):
		cls.background = LoadImage("Backgrounds", "background.png")
		cls.hamsters = LoadImage("Backgrounds", "hamsters.png")
		cls.highscore = LoadImage("Backgrounds", "highscore.png")
		cls.rules = LoadImage("Backgrounds", "rules.png")
		cls.title = LoadImage("Backgrounds", "title.png")


class SoundController:
	
	@classmethod
	def Load(cls):
		cls.transformLeft = LoadSound("Sounds", "transform_left.wav")
		cls.transformRight = LoadSound("Sounds", "transform_right.wav")
		cls.error = LoadSound("Sounds", "error.wav")
		cls.gameOver = LoadSound("Sounds", "gameover.wav")
		cls.victory = LoadSound("Sounds", "multiply.wav")
		cls.blob = LoadSound("Sounds", "blob.wav")
		LoadMusic("Sounds", "bgmusic.ogg")


class TextureController:
	
	@classmethod
	def Load(cls):
		cls.selector = LoadImage("Textures", "selector.png")
		cls.cycleBar = LoadImage("Textures", "cyclebar.png")
		cls.levelComplete = LoadImage("Textures", "level_complete.png")
		cls.plague = LoadImage("Textures", "plague.png")
		cls.digitsAtlas = LoadImage("Textures", "atlas_score.png")
		cls.doctorAtlas = LoadImage("Textures", "atlas_doctor.png")
		cls.femaleAtlas = LoadImage("Textures", "atlas_female.png")
		cls.maleAtlas = LoadImage("Textures", "atlas_male.png")
		
	@classmethod
	def ScaleHamsters(cls, size):
		cls.scaledPlague = pygame.transform.smoothscale(cls.plague, (size, size))
		cls.scaledDoctorAtlas = pygame.transform.smoothscale(cls.doctorAtlas, (size * ATLAS_LENGTH, size))
		cls.scaledFemaleAtlas = pygame.transform.smoothscale(cls.femaleAtlas, (size * ATLAS_LENGTH, size))
		cls.scaledMaleAtlas = pygame.transform.smoothscale(cls.maleAtlas, (size * ATLAS_LENGTH, size))
		

class Level:
	
	@classmethod
	def CountLevels(cls):
		cls.levelsAvailable = len(pkg_resources.resource_listdir("HamstersAndPlague", "Levels"))
		
	def __init__(self, levelNumber):
		levelFile = pkg_resources.resource_stream("HamstersAndPlague", "Levels/level_%s.rpl" % levelNumber)
		self.target = int(levelFile.readline())
		self.cycle = int(levelFile.readline())
		self.rows = int(levelFile.readline())
		self.cols = int(levelFile.readline())
		
		self.layout = []
		
		for r in range(self.rows):
			row = levelFile.readline().rstrip().split(" ")
			self.layout.append(row)
			
class HighScore:
		
	def __init__(self):
		self.highScore = []
		
	def load(self):
		try:
			f = open(os.path.expanduser(os.path.join("~", "hamstersdata", "highscore.txt")))
		except IOError:
			self.highScore.extend([0,0,0,0,0])
		else:	
			lines = f.readlines()
			for l in lines:
				self.highScore.append(int(l))
				
			f.close()
			
			
	def save(self):
		try:
			f = open(os.path.expanduser(os.path.join("~", "hamstersdata", "highscore.txt")), "w")
		except IOError:
			try:
				os.mkdir(os.path.expanduser(os.path.join("~", "hamstersdata")))
				f = open(os.path.expanduser(os.path.join("~", "hamstersdata", "highscore.txt")), "w")
			except:
				pass
		else:	
			for s in self.highScore:
				f.write(str(s) + "\n")
			
			f.close()
		
		self.highScore = []	
		
