# -*- encoding: utf-8 -*-

import pygame
from HamstersAndPlague.Constants import *

class Critter(pygame.sprite.DirtySprite):
	
	immune = False
	
	def __init__(self, atlas, age):
		pygame.sprite.DirtySprite.__init__(self)
		self.atlas = atlas
		self.age = age
		
	def Place(self, grid, row, col, size):
		self.grid = grid
		self.row = row
		self.col = col
		x = col * size + XMARGIN_LEFT
		y = row * size + YMARGIN
		
		self.image = self.atlas.subsurface(pygame.Rect(self.age * size, 0, size, size))
		self.rect = pygame.Rect(x, y, size, size)
		

	def Move(self, rowAdjust, colAdjust):
		self.row += rowAdjust
		self.col += colAdjust
		self.rect.left += colAdjust * self.rect.width
		self.rect.top += rowAdjust * self.rect.height

	def Age(self):
		self.age += 1
		self.image = self.atlas.subsurface(pygame.Rect(self.age * self.rect.width, 0, self.rect.width, self.rect.height))
		
		if self.age > DEATH:
			self.grid.ClearTile(self.row, self.col)
			return True
		else:
			return False
		
	


class Male(Critter):
	pass


class Female(Critter):
	
	def CheckMates(self):
		if self.age == ADULT:
			neighbours = self.grid.GetNeighbours(self.row, self.col)
			for n in neighbours:
				if type(n) == Male and n.age == ADULT:
					self.grid.SpawnFromTile(self.row, self.col)
					break

class Doctor(Critter):
	
	def Age(self):
		if self.age > 0:
			self.immune = True
			
		return Critter.Age(self)
			
		

class Plagued(Critter):
	
	immune = True
	
	def __init__(self, texture, age):
		Critter.__init__(self, texture, age)
		self.image = texture
		
	def Place(self, grid, row, col, size):
		Critter.Place(self, grid, row, col, size)
		
	def SpreadPlague(self):
		neighbours = self.grid.GetNeighbours(self.row, self.col)
		for n in neighbours:
			if n and not n.immune:
				self.grid.ReplaceWithPlagued(n.row, n.col)
		
		