# -*- encoding: utf-8 -*-

import pygame
from HamstersAndPlague.Selector import TwoByTwoSelector
from HamstersAndPlague.ContentControllers import TextureController
from HamstersAndPlague.Constants import *

class Grid:
	
	def __init__(self, critterManager):
		self.critterManager = critterManager
		self.width = SCREEN_WIDTH - XMARGIN_LEFT - XMARGIN_RIGHT
		
	def LoadLevel(self, level):
		self.grid = [[None for c in range(level.cols)] for r in range(level.rows)]
		self.tileSize = self.width / level.cols
		self.cols = level.cols
		self.rows = level.rows
		TextureController.ScaleHamsters(self.tileSize)
		
		for r, row in enumerate(level.layout):
			for c, col in enumerate(row): 
				if col != "--":
					self._PlaceCritter(col[0], int(col[1]), r, c)
					
		self._PlaceSelector(level.rows / 2, level.cols / 2)
		self.plagueCount = 0
		self.spawnCount = 0
					
	def GetSelection(self, row, col, width, height):
		selection = []
		for c in range(col, col + width):
			for r in range(row, row + height):
				selection.append(self.grid[r][c])
				self.grid[r][c] = None
				
		return selection
			
	def GetNeighbours(self, row, col):
		neighbours = []
		if row > 0: 
			neighbours.append(self.grid[row-1][col])
		if row < self.rows - 1: 
			neighbours.append(self.grid[row+1][col])
		if col > 0:
			neighbours.append(self.grid[row][col-1])
		if col < self.cols - 1:
			neighbours.append(self.grid[row][col+1])
		return neighbours
			
	def MoveCritters(self, critters):
		for c in critters:
			if c:
				self.grid[c.row][c.col] = c
									
	def ReplaceWithPlagued(self, row, col):
		self.critterManager.RemovePlagued(self.grid[row][col])
		p = self._PlaceCritter("P", 0, row, col)
		self.plagueCount += 1
											
	def SpawnFromTile(self, row, col):
		for r in range(row-1, row+2):
			if r >= 0 and r < self.rows:
				for c in range(col-1, col+2):
					if c >= 0 and c < self.cols and not self.grid[r][c]:
						self._PlaceCritter("R", 0, r, c)
						self.spawnCount += 1
												
	
	
	def GetCycleScore(self):
		score = self.spawnCount * self.spawnCount - self.plagueCount * PLAGUE_PENALTY
		self.spawnCount = 0
		self.plagueCount = 0
		return score
	
	def _PlaceCritter(self, cType, cAge, row, col):
		critter = self.critterManager.SpawnCritter(cType, cAge)
		critter.Place(self, row, col, self.tileSize)
		self.grid[row][col] = critter
		
	def _PlaceSelector(self, row, col):
		self.selector = TwoByTwoSelector(self, self.tileSize, row, col)
		
	def CreateSprites(self, group):
		group.add(self.selector)
		
	def MousePosition(self, (mouseX, mouseY)):
		mouseX = mouseX - XMARGIN_LEFT
		col = int(mouseX / self.tileSize + 0.5)
		self.selector.MouseMoveX(col, self.cols)
		
		mouseY = mouseY - YMARGIN
		row = int(mouseY / self.tileSize + 0.5)
		self.selector.MouseMoveY(row, self.rows)
		
	def ClearTile(self, row, col):
		self.grid[row][col] = None