# -*- encoding: utf-8 -*-

import pygame
from HamstersAndPlague.Constants import *

from HamstersAndPlague.ContentControllers import *
from HamstersAndPlague.CritterManager import CritterManager
from HamstersAndPlague.Overlays import CycleCounter, DigitString
from HamstersAndPlague.Popups import StaticPopup
from HamstersAndPlague.Grid import Grid

class HamstersMain:
	
	def __init__(self):
		self.window = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT))
		self._LoadContent()
		self.critterManager = CritterManager()
		self.cycleCounter = CycleCounter()
		self.totalScoreString = DigitString(SCORE_DIGITS, TOTALSCORE_X, TOTALSCORE_Y)
		self.currentCyclesString = DigitString(CYCLE_DIGITS, CURRENTCYCLES_X, CURRENTCYCLES_Y)
		self.targetCyclesString = DigitString(CYCLE_DIGITS, TARGETCYCLES_X, TARGETCYCLES_Y)
		self.grid = Grid(self.critterManager)
		self.bg = pygame.sprite.DirtySprite()
		self.bg.rect = pygame.Rect(0, 0, 800, 600)
		self.bgGroup = pygame.sprite.RenderUpdates()
		self.hamsterGroup = pygame.sprite.RenderUpdates()
		self.overlaysGroup = pygame.sprite.RenderUpdates()
		self.popupGroup = pygame.sprite.RenderUpdates()
		self.delayTimer = 65536
		
		self.bgGroup.add(self.bg)
		self.highScore = HighScore()
		
		self._TitleScreen()
		
	
	def _LoadContent(self):
		BackgroundController.Load()
		SoundController.Load()
		TextureController.Load()
			
	def Update(self, ticks):
		
		if self.delayTimer == 65536:
		
			if self.state == PLAYING:
				self.cycleTimer -= ticks
				self.cycleCounter.Decrement(float(self.cycleTimer) / self.cycleLength)
				if self.cycleTimer <= 0.0:
					self.cycleTimer = self.cycleLength
					self.cycleCounter.Reset()
					SoundController.blob.play()
					
					if self.critterManager.ManageCritters():
						SoundController.gameOver.play()
						self._GameOver()
						return
				
					self.cycles += 1
					self.currentCyclesString.SetValue(self.cycles, self.bg.image)
					
					self.totalScore += self.grid.GetCycleScore()
				
					if self.totalScore < 0: 
						self.totalScore = 0
				
					self.totalScoreString.SetValue(self.totalScore, self.bg.image)
				
					if self.cycles == self.target:
						self.cycles = 0
						SoundController.victory.play()
						self.totalScore += self.currentLevel * LEVEL_COMPLETE_BONUS
						self.delayTimer = LEVEL_LOAD_WAIT
						self.popupGroup.add(StaticPopup(TextureController.levelComplete, LEVEL_COMPLETE_X, LEVEL_COMPLETE_Y))
						if self.currentLevel < Level.levelsAvailable:
							self.currentLevel += 1
							self._NewLevel()
						else:
							self._GameOver()
					
				self.grid.MousePosition(pygame.mouse.get_pos())
		
		elif self.delayTimer <= 0:
			self.popupGroup.empty()
			self.delayTimer = 65536
		
		elif self.delayTimer > 0:
			self.delayTimer -= ticks
			
			
	def Draw(self):
		update = []
		
		update.extend(self.bgGroup.draw(self.window))
		
		if self.state == TITLE:
			pass
		
		elif self.state == GAMEOVER:
			update.extend(self.overlaysGroup.draw(self.window))
		
		elif self.state == PLAYING:
			update.extend(self.hamsterGroup.draw(self.window))
			update.extend(self.overlaysGroup.draw(self.window))			
			update.extend(self.popupGroup.draw(self.window))
		
		pygame.display.update(update)
	
	
	def MousePress(self, buttons):
		if self.delayTimer == 65536:
		
			if self.state == TITLE:
				self._HelpPage1()
			
			elif self.state == HELP1:
				self._HelpPage2()
				
			elif self.state == HELP2:
				self._NewGame()
			
			elif self.state == GAMEOVER:
				self._TitleScreen()
			
			elif self.state == PLAYING:
				if buttons[0]:
					self.grid.selector.Transform(LEFT)
				elif buttons[2]:
					self.grid.selector.Transform(RIGHT)
				
	
	def _NewLevel(self):
		self.hamsterGroup.empty()
		self.overlaysGroup.empty()
		self.currentCyclesString.SetValue(0, self.bg.image)
		self.critterManager.Reset(self.hamsterGroup)
		level = Level(self.currentLevel)
		self.grid.LoadLevel(level)
		self.target = level.target
		self.targetCyclesString.SetValue(self.target, self.bg.image)
		self.cycleLength = level.cycle
		self.cycles = 0
		self.cycleTimer = self.cycleLength
		self.cycleCounter.Reset()
		self.overlaysGroup.add(self.cycleCounter)
		self.overlaysGroup.add(self.grid.selector)
		self.overlaysGroup.add(self.currentCyclesString)
		self.overlaysGroup.add(self.targetCyclesString)
		self.overlaysGroup.add(self.totalScoreString)
		
		
		
	def _NewGame(self):
		pygame.mixer.music.play(-1)
		Level.CountLevels()
		self.currentLevel = 1
		self._PlayingGame()
		self._NewLevel()
		self.totalScore = 0
		self.totalScoreString.SetValue(0, self.bg.image)
		
		
	
	def _GameOver(self):
		self.hamsterGroup.empty()
		self.overlaysGroup.empty()
		self.bg.image = BackgroundController.highscore
		self.bg.dirty = 1
		
		self.highScore.load()
		self.highScore.highScore.append(self.totalScore)
		self.highScore.highScore.sort()
		self.highScore.highScore.reverse()
		
		result = DigitString(FINAL_SCORE_DIGITS, FINALSCORE_X, FINALSCORE_Y)
		result.SetValue(self.totalScore, self.bg.image)
		self.overlaysGroup.add(result)
		
		if len(self.highScore.highScore) > 5:
			self.highScore.highScore = self.highScore.highScore[:5]
			
		for i, score in enumerate(self.highScore.highScore):
			scoreString = DigitString(FINAL_SCORE_DIGITS, HIGHSCORE_TOP_X, HIGHSCORE_TOP_Y + i * HIGHSCORE_Y_SPACING)
			scoreString.SetValue(score, self.bg.image)
			self.overlaysGroup.add(scoreString)
			
		pygame.mixer.music.stop()		
		self.state = GAMEOVER
		self.highScore.save()
		pygame.mouse.set_visible(True)
	
	def _PlayingGame(self):
		self.bg.image = BackgroundController.background
		self.bg.dirty = 1
		self.state = PLAYING
		pygame.mouse.set_visible(False)
	
	def _HelpPage1(self):
		self.bg.image = BackgroundController.hamsters
		self.dirty = 1
		self.state = HELP1
		pygame.mouse.set_visible(True)
	
	def _HelpPage2(self):
		self.bg.image = BackgroundController.rules
		self.dirty = 1
		self.state = HELP2
		pygame.mouse.set_visible(True)
	
	def _TitleScreen(self):
		self.bg.image = BackgroundController.title
		self.bg.dirty = 1
		self.state = TITLE
		pygame.mouse.set_visible(True)
	
		