# -*- encoding: utf-8 -*-

import pygame
from HamstersAndPlague.Critters import Plagued
from HamstersAndPlague.ContentControllers import TextureController, SoundController
from HamstersAndPlague.Constants import *

class BaseSelector(pygame.sprite.DirtySprite):
	pass


class TwoByTwoSelector(BaseSelector):

	width = 2
	height = 2

	rightTransform = ( (0, 1), (-1, 0), (1, 0), (0, -1) )
	leftTransform = ( (1, 0), (0, 1), (0 ,-1), (-1, 0) )

	def __init__(self, grid, size, row, col):
		pygame.sprite.DirtySprite.__init__(self)
		self.grid = grid
		self.row = row
		self.col = col
		
		x = col * size + XMARGIN_LEFT
		y = row * size + YMARGIN		
		self.rect = pygame.Rect(x, y, size * self.width, size * self.height)
		self.image = pygame.transform.smoothscale(TextureController.selector, (size * self.width, size * self.height))
		self.moved = False
		
	def Transform(self, direction):
		critters = self.grid.GetSelection(self.row, self.col, self.width, self.height)
		
		canTransform = True
		for c in critters:
			if type(c) == Plagued:
				canTransform = False
				break
		
		if canTransform:
			sound = False
		
			for i, c in enumerate(critters): 
				if c: 
					if direction == LEFT:
						c.Move(self.leftTransform[i][0], self.leftTransform[i][1])
					elif direction == RIGHT:
						c.Move(self.rightTransform[i][0], self.rightTransform[i][1])
					
					sound = True
			
			if sound:
				if direction == LEFT:
					SoundController.transformLeft.play()
				elif direction == RIGHT:
					SoundController.transformRight.play()
			
		else:
			SoundController.error.play()
			
		self.grid.MoveCritters(critters)
	
	def MouseMoveX(self, col, cols):
		if col >= 0 and col <= cols - 2:
			originCol = col
		elif col < 0:
			originCol = 0
		elif col > cols - 2:
			originCol = cols - 2
		
		if originCol != self.col:
			self.rect.left = originCol * self.rect.width / 2 + XMARGIN_LEFT
			self.col = originCol
			self.dirty = 1
	
	def MouseMoveY(self, row, rows):
		if row >= 0 and row <= rows - 2:
			originRow = row
		elif row < 0:
			originRow = 0
		elif row > rows - 2:
			originRow = rows - 2
		
		if originRow != self.row: 
			self.rect.top = originRow * self.rect.width / 2 + YMARGIN
			self.row = originRow
			self.dirty = 1

